# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from datetime import datetime
from typing import Optional, Union, TYPE_CHECKING

from bungio.utils import enum_converter
from bungio.models.base import BaseModel, BaseEnum, custom_define, custom_field

from bungio.models.mixins import DestinyCharacterMixin

if TYPE_CHECKING:
    from bungio.models import DestinyActivityDefinition
    from bungio.models import BungieMembershipType
    from bungio.models import DestinyFireteamFinderActivityGraphDefinition


class DestinyFireteamFinderApplicationType(BaseEnum):
    """
    _No description given by bungie._
    """

    UNKNOWN = 0
    """_No description given by bungie._ """
    CREATOR = 1
    """_No description given by bungie._ """
    SEARCH = 2
    """_No description given by bungie._ """
    INVITE = 3
    """_No description given by bungie._ """
    FRIEND = 4
    """_No description given by bungie._ """
    ENCOUNTER = 5
    """_No description given by bungie._ """
    PUBLIC = 6
    """_No description given by bungie._ """


@custom_define()
class DestinyFireteamFinderApplyToListingResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        application: _No description given by bungie._
        is_applied: _No description given by bungie._
        listing: _No description given by bungie._
    """

    application: "DestinyFireteamFinderApplication" = custom_field()
    is_applied: bool = custom_field()
    listing: "DestinyFireteamFinderListing" = custom_field()


@custom_define()
class DestinyFireteamFinderApplication(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        applicant_set: _No description given by bungie._
        application_id: _No description given by bungie._
        application_type: _No description given by bungie._
        created_date_time: _No description given by bungie._
        listing_id: _No description given by bungie._
        referral_token: _No description given by bungie._
        revision: _No description given by bungie._
        state: _No description given by bungie._
        submitter_id: _No description given by bungie._
    """

    applicant_set: "DestinyFireteamFinderApplicantSet" = custom_field()
    application_id: int = custom_field(metadata={"int64": True})
    application_type: Union["DestinyFireteamFinderApplicationType", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderApplicationType")
    )
    created_date_time: datetime = custom_field()
    listing_id: int = custom_field(metadata={"int64": True})
    referral_token: int = custom_field(metadata={"int64": True})
    revision: int = custom_field()
    state: Union["DestinyFireteamFinderApplicationState", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderApplicationState")
    )
    submitter_id: "DestinyFireteamFinderPlayerId" = custom_field()


class DestinyFireteamFinderApplicationState(BaseEnum):
    """
    _No description given by bungie._
    """

    UNKNOWN = 0
    """_No description given by bungie._ """
    WAITING_FOR_APPLICANTS = 1
    """_No description given by bungie._ """
    WAITING_FOR_LOBBY_OWNER = 2
    """_No description given by bungie._ """
    ACCEPTED = 3
    """_No description given by bungie._ """
    REJECTED = 4
    """_No description given by bungie._ """
    DELETED = 5
    """_No description given by bungie._ """
    EXPIRED = 6
    """_No description given by bungie._ """


@custom_define()
class DestinyFireteamFinderPlayerId(BaseModel, DestinyCharacterMixin):
    """
    _No description given by bungie._

    None
    Attributes:
        character_id: _No description given by bungie._
        membership_id: _No description given by bungie._
        membership_type: _No description given by bungie._
    """

    character_id: int = custom_field(metadata={"int64": True})
    membership_id: int = custom_field(metadata={"int64": True})
    membership_type: Union["BungieMembershipType", int] = custom_field(converter=enum_converter("BungieMembershipType"))


@custom_define()
class DestinyFireteamFinderApplicantSet(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        applicants: _No description given by bungie._
    """

    applicants: list[dict] = custom_field(metadata={"type": """list[dict]"""})


@custom_define()
class DestinyFireteamFinderListing(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        available_slots: _No description given by bungie._
        created_date_time: _No description given by bungie._
        listing_id: _No description given by bungie._
        lobby_id: _No description given by bungie._
        lobby_state: _No description given by bungie._
        owner_id: _No description given by bungie._
        revision: _No description given by bungie._
        settings: _No description given by bungie._
    """

    available_slots: int = custom_field()
    created_date_time: datetime = custom_field()
    listing_id: int = custom_field(metadata={"int64": True})
    lobby_id: int = custom_field(metadata={"int64": True})
    lobby_state: Union["DestinyFireteamFinderLobbyState", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderLobbyState")
    )
    owner_id: "DestinyFireteamFinderPlayerId" = custom_field()
    revision: int = custom_field()
    settings: "DestinyFireteamFinderLobbySettings" = custom_field()


@custom_define()
class DestinyFireteamFinderLobbySettings(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_graph_hash: _No description given by bungie._
        activity_hash: _No description given by bungie._
        clan_id: _No description given by bungie._
        listing_values: _No description given by bungie._
        max_player_count: _No description given by bungie._
        online_players_only: _No description given by bungie._
        privacy_scope: _No description given by bungie._
        scheduled_date_time: _No description given by bungie._
        manifest_activity_graph_hash: Manifest information for `activity_graph_hash`
        manifest_activity_hash: Manifest information for `activity_hash`
    """

    activity_graph_hash: int = custom_field()
    activity_hash: int = custom_field()
    clan_id: int = custom_field(metadata={"int64": True})
    listing_values: list["DestinyFireteamFinderListingValue"] = custom_field(
        metadata={"type": """list[DestinyFireteamFinderListingValue]"""}
    )
    max_player_count: int = custom_field()
    online_players_only: bool = custom_field()
    privacy_scope: Union["DestinyFireteamFinderLobbyPrivacyScope", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderLobbyPrivacyScope")
    )
    scheduled_date_time: datetime = custom_field()
    manifest_activity_graph_hash: Optional["DestinyFireteamFinderActivityGraphDefinition"] = custom_field(default=None)
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)


class DestinyFireteamFinderLobbyPrivacyScope(BaseEnum):
    """
    _No description given by bungie._
    """

    UNKNOWN = 0
    """_No description given by bungie._ """
    OPEN = 1
    """_No description given by bungie._ """
    APPLICATIONS = 2
    """_No description given by bungie._ """
    CLAN = 3
    """_No description given by bungie._ """
    FRIENDS = 4
    """_No description given by bungie._ """


@custom_define()
class DestinyFireteamFinderListingValue(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        value_type: _No description given by bungie._
        values: _No description given by bungie._
    """

    value_type: int = custom_field()
    values: list[int] = custom_field(metadata={"type": """list[int]"""})


class DestinyFireteamFinderLobbyState(BaseEnum):
    """
    _No description given by bungie._
    """

    UNKNOWN = 0
    """_No description given by bungie._ """
    INACTIVE = 1
    """_No description given by bungie._ """
    ACTIVE = 2
    """_No description given by bungie._ """
    EXPIRED = 3
    """_No description given by bungie._ """
    CLOSED = 4
    """_No description given by bungie._ """
    CANCELED = 5
    """_No description given by bungie._ """
    DELETED = 6
    """_No description given by bungie._ """


@custom_define()
class DestinyFireteamFinderBulkGetListingStatusResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        listing_status: _No description given by bungie._
    """

    listing_status: list["DestinyFireteamFinderListingStatus"] = custom_field(
        metadata={"type": """list[DestinyFireteamFinderListingStatus]"""}
    )


@custom_define()
class DestinyFireteamFinderListingStatus(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        available_slots: _No description given by bungie._
        listing_id: _No description given by bungie._
        listing_revision: _No description given by bungie._
    """

    available_slots: int = custom_field()
    listing_id: int = custom_field(metadata={"int64": True})
    listing_revision: int = custom_field()


@custom_define()
class DestinyFireteamFinderLobbyListingReference(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        listing_id: _No description given by bungie._
        lobby_id: _No description given by bungie._
    """

    listing_id: int = custom_field(metadata={"int64": True})
    lobby_id: int = custom_field(metadata={"int64": True})


@custom_define()
class DestinyFireteamFinderGetApplicationResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        applicant_set: _No description given by bungie._
        application_id: _No description given by bungie._
        application_type: _No description given by bungie._
        created_date_time: _No description given by bungie._
        listing_id: _No description given by bungie._
        referral_token: _No description given by bungie._
        revision: _No description given by bungie._
        state: _No description given by bungie._
        submitter_id: _No description given by bungie._
    """

    applicant_set: "DestinyFireteamFinderApplicantSet" = custom_field()
    application_id: int = custom_field(metadata={"int64": True})
    application_type: Union["DestinyFireteamFinderApplicationType", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderApplicationType")
    )
    created_date_time: datetime = custom_field()
    listing_id: int = custom_field(metadata={"int64": True})
    referral_token: int = custom_field(metadata={"int64": True})
    revision: int = custom_field()
    state: Union["DestinyFireteamFinderApplicationState", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderApplicationState")
    )
    submitter_id: "DestinyFireteamFinderPlayerId" = custom_field()


@custom_define()
class DestinyFireteamFinderGetListingApplicationsResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        applications: _No description given by bungie._
        next_page_token: _No description given by bungie._
        page_size: _No description given by bungie._
    """

    applications: list["DestinyFireteamFinderApplication"] = custom_field(
        metadata={"type": """list[DestinyFireteamFinderApplication]"""}
    )
    next_page_token: str = custom_field()
    page_size: int = custom_field()


@custom_define()
class DestinyFireteamFinderLobbyResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        created_date_time: _No description given by bungie._
        listing_id: _No description given by bungie._
        lobby_id: _No description given by bungie._
        owner: _No description given by bungie._
        players: _No description given by bungie._
        revision: _No description given by bungie._
        settings: _No description given by bungie._
        state: _No description given by bungie._
    """

    created_date_time: datetime = custom_field()
    listing_id: int = custom_field(metadata={"int64": True})
    lobby_id: int = custom_field(metadata={"int64": True})
    owner: "DestinyFireteamFinderPlayerId" = custom_field()
    players: list["DestinyFireteamFinderLobbyPlayer"] = custom_field(
        metadata={"type": """list[DestinyFireteamFinderLobbyPlayer]"""}
    )
    revision: int = custom_field()
    settings: "DestinyFireteamFinderLobbySettings" = custom_field()
    state: Union["DestinyFireteamFinderLobbyState", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderLobbyState")
    )


@custom_define()
class DestinyFireteamFinderLobbyPlayer(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        offer_id: _No description given by bungie._
        player_id: _No description given by bungie._
        referral_token: _No description given by bungie._
        state: _No description given by bungie._
    """

    offer_id: int = custom_field(metadata={"int64": True})
    player_id: "DestinyFireteamFinderPlayerId" = custom_field()
    referral_token: int = custom_field(metadata={"int64": True})
    state: Union["DestinyFireteamFinderPlayerReadinessState", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderPlayerReadinessState")
    )


class DestinyFireteamFinderPlayerReadinessState(BaseEnum):
    """
    _No description given by bungie._
    """

    UNKNOWN = 0
    """_No description given by bungie._ """
    RESERVED = 1
    """_No description given by bungie._ """
    DISCONNECTED = 2
    """_No description given by bungie._ """
    IN_LOBBY_UNREADY = 3
    """_No description given by bungie._ """
    IN_LOBBY_READY = 4
    """_No description given by bungie._ """
    SUMMONED = 5
    """_No description given by bungie._ """


@custom_define()
class DestinyFireteamFinderGetPlayerLobbiesResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        lobbies: All available lobbies that this player has created or is a member of.
        next_page_token: A string token required to get the next page of results. This will be null or empty if there are no more results.
        page_size: The number of results requested.
    """

    lobbies: list["DestinyFireteamFinderLobbyResponse"] = custom_field(
        metadata={"type": """list[DestinyFireteamFinderLobbyResponse]"""}
    )
    next_page_token: str = custom_field()
    page_size: int = custom_field()


@custom_define()
class DestinyFireteamFinderGetPlayerApplicationsResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        applications: All applications that this player has sent.
        next_page_token: String token to request next page of results.
    """

    applications: list["DestinyFireteamFinderApplication"] = custom_field(
        metadata={"type": """list[DestinyFireteamFinderApplication]"""}
    )
    next_page_token: str = custom_field()


@custom_define()
class DestinyFireteamFinderGetPlayerOffersResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        offers: All offers that this player has recieved.
    """

    offers: list["DestinyFireteamFinderOffer"] = custom_field(metadata={"type": """list[DestinyFireteamFinderOffer]"""})


@custom_define()
class DestinyFireteamFinderOffer(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        application_id: _No description given by bungie._
        created_date_time: _No description given by bungie._
        lobby_id: _No description given by bungie._
        offer_id: _No description given by bungie._
        revision: _No description given by bungie._
        state: _No description given by bungie._
        target_id: _No description given by bungie._
    """

    application_id: int = custom_field(metadata={"int64": True})
    created_date_time: datetime = custom_field()
    lobby_id: int = custom_field(metadata={"int64": True})
    offer_id: int = custom_field(metadata={"int64": True})
    revision: int = custom_field()
    state: Union["DestinyFireteamFinderOfferState", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderOfferState")
    )
    target_id: "DestinyFireteamFinderPlayerId" = custom_field()


class DestinyFireteamFinderOfferState(BaseEnum):
    """
    _No description given by bungie._
    """

    UNKNOWN = 0
    """_No description given by bungie._ """
    PENDING = 1
    """_No description given by bungie._ """
    ACCEPTED = 2
    """_No description given by bungie._ """
    REJECTED = 3
    """_No description given by bungie._ """
    DELETED = 4
    """_No description given by bungie._ """
    EXPIRED = 5
    """_No description given by bungie._ """


@custom_define()
class DestinyFireteamFinderGetCharacterActivityAccessResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        fireteam_finder_activity_graph_states: A map of fireteam finder activity graph hashes to visibility and availability states.
    """

    fireteam_finder_activity_graph_states: dict[int, "DestinyFireteamFinderActivityGraphState"] = custom_field(
        metadata={"type": """dict[int, DestinyFireteamFinderActivityGraphState]"""}
    )


@custom_define()
class DestinyFireteamFinderActivityGraphState(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        is_available: Indicates if this fireteam finder activity graph node is available to select for this character.
        is_visible: Indicates if this fireteam finder activity graph node is visible for this character.
    """

    is_available: bool = custom_field()
    is_visible: bool = custom_field()


@custom_define()
class DestinyFireteamFinderGetLobbyOffersResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        offers: _No description given by bungie._
        page_token: _No description given by bungie._
    """

    offers: list["DestinyFireteamFinderOffer"] = custom_field(metadata={"type": """list[DestinyFireteamFinderOffer]"""})
    page_token: str = custom_field()


@custom_define()
class DestinyFireteamFinderHostLobbyResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        application_id: _No description given by bungie._
        listing_id: _No description given by bungie._
        lobby_id: _No description given by bungie._
        offer_id: _No description given by bungie._
    """

    application_id: int = custom_field(metadata={"int64": True})
    listing_id: int = custom_field(metadata={"int64": True})
    lobby_id: int = custom_field(metadata={"int64": True})
    offer_id: int = custom_field(metadata={"int64": True})


@custom_define()
class DestinyFireteamFinderHostLobbyRequest(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_graph_hash: _No description given by bungie._
        activity_hash: _No description given by bungie._
        clan_id: _No description given by bungie._
        listing_values: _No description given by bungie._
        max_player_count: _No description given by bungie._
        online_players_only: _No description given by bungie._
        privacy_scope: _No description given by bungie._
        scheduled_date_time: _No description given by bungie._
        manifest_activity_graph_hash: Manifest information for `activity_graph_hash`
        manifest_activity_hash: Manifest information for `activity_hash`
    """

    activity_graph_hash: int = custom_field()
    activity_hash: int = custom_field()
    clan_id: int = custom_field(metadata={"int64": True})
    listing_values: list["DestinyFireteamFinderListingValue"] = custom_field(
        metadata={"type": """list[DestinyFireteamFinderListingValue]"""}
    )
    max_player_count: int = custom_field()
    online_players_only: bool = custom_field()
    privacy_scope: Union["DestinyFireteamFinderLobbyPrivacyScope", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderLobbyPrivacyScope")
    )
    scheduled_date_time: datetime = custom_field()
    manifest_activity_graph_hash: Optional["DestinyFireteamFinderActivityGraphDefinition"] = custom_field(default=None)
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)


@custom_define()
class DestinyFireteamFinderJoinLobbyRequest(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        lobby_id: _No description given by bungie._
        offer_id: _No description given by bungie._
    """

    lobby_id: int = custom_field(metadata={"int64": True})
    offer_id: int = custom_field(metadata={"int64": True})


@custom_define()
class DestinyFireteamFinderKickPlayerRequest(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        target_character_id: _No description given by bungie._
        target_membership_type: _No description given by bungie._
    """

    target_character_id: int = custom_field(metadata={"int64": True})
    target_membership_type: Union["BungieMembershipType", int] = custom_field(
        converter=enum_converter("BungieMembershipType")
    )


@custom_define()
class DestinyFireteamFinderRespondToApplicationResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        application_id: _No description given by bungie._
        application_revision: _No description given by bungie._
    """

    application_id: int = custom_field(metadata={"int64": True})
    application_revision: int = custom_field()


@custom_define()
class DestinyFireteamFinderRespondToApplicationRequest(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        accepted: _No description given by bungie._
    """

    accepted: bool = custom_field()


@custom_define()
class DestinyFireteamFinderRespondToAuthenticationResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        application_id: _No description given by bungie._
        application_revision: _No description given by bungie._
        listing: _No description given by bungie._
        offer: _No description given by bungie._
    """

    application_id: int = custom_field(metadata={"int64": True})
    application_revision: int = custom_field()
    listing: "DestinyFireteamFinderListing" = custom_field()
    offer: "DestinyFireteamFinderOffer" = custom_field()


@custom_define()
class DestinyFireteamFinderRespondToAuthenticationRequest(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        confirmed: _No description given by bungie._
    """

    confirmed: bool = custom_field()


@custom_define()
class DestinyFireteamFinderRespondToOfferResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        offer_id: _No description given by bungie._
        revision: _No description given by bungie._
        state: _No description given by bungie._
    """

    offer_id: int = custom_field(metadata={"int64": True})
    revision: int = custom_field()
    state: Union["DestinyFireteamFinderOfferState", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderOfferState")
    )


@custom_define()
class DestinyFireteamFinderRespondToOfferRequest(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        accepted: _No description given by bungie._
    """

    accepted: bool = custom_field()


@custom_define()
class DestinyFireteamFinderSearchListingsByClanResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        listings: _No description given by bungie._
        page_token: _No description given by bungie._
    """

    listings: list["DestinyFireteamFinderListing"] = custom_field(
        metadata={"type": """list[DestinyFireteamFinderListing]"""}
    )
    page_token: str = custom_field()


@custom_define()
class DestinyFireteamFinderSearchListingsByClanRequest(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        lobby_state: _No description given by bungie._
        page_size: _No description given by bungie._
        page_token: _No description given by bungie._
    """

    lobby_state: Union["DestinyFireteamFinderLobbyState", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderLobbyState")
    )
    page_size: int = custom_field()
    page_token: str = custom_field()


@custom_define()
class DestinyFireteamFinderSearchListingsByFiltersResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        listings: _No description given by bungie._
        page_token: _No description given by bungie._
    """

    listings: list["DestinyFireteamFinderListing"] = custom_field(
        metadata={"type": """list[DestinyFireteamFinderListing]"""}
    )
    page_token: str = custom_field()


@custom_define()
class DestinyFireteamFinderSearchListingsByFiltersRequest(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        filters: _No description given by bungie._
        lobby_state: _No description given by bungie._
        page_size: _No description given by bungie._
        page_token: _No description given by bungie._
    """

    filters: list["DestinyFireteamFinderListingFilter"] = custom_field(
        metadata={"type": """list[DestinyFireteamFinderListingFilter]"""}
    )
    lobby_state: Union["DestinyFireteamFinderLobbyState", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderLobbyState")
    )
    page_size: int = custom_field()
    page_token: str = custom_field()


@custom_define()
class DestinyFireteamFinderListingFilter(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        listing_value: _No description given by bungie._
        match_type: _No description given by bungie._
        range_type: _No description given by bungie._
    """

    listing_value: "DestinyFireteamFinderListingValue" = custom_field()
    match_type: Union["DestinyFireteamFinderListingFilterMatchType", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderListingFilterMatchType")
    )
    range_type: Union["DestinyFireteamFinderListingFilterRangeType", int] = custom_field(
        converter=enum_converter("DestinyFireteamFinderListingFilterRangeType")
    )


class DestinyFireteamFinderListingFilterRangeType(BaseEnum):
    """
    _No description given by bungie._
    """

    UNKNOWN = 0
    """_No description given by bungie._ """
    ALL = 1
    """_No description given by bungie._ """
    ANY = 2
    """_No description given by bungie._ """
    IN_RANGE_INCLUSIVE = 3
    """_No description given by bungie._ """
    IN_RANGE_EXCLUSIVE = 4
    """_No description given by bungie._ """
    GREATER_THAN = 5
    """_No description given by bungie._ """
    GREATER_THAN_OR_EQUAL_TO = 6
    """_No description given by bungie._ """
    LESS_THAN = 7
    """_No description given by bungie._ """
    LESS_THAN_OR_EQUAL_TO = 8
    """_No description given by bungie._ """


class DestinyFireteamFinderListingFilterMatchType(BaseEnum):
    """
    _No description given by bungie._
    """

    UNKNOWN = 0
    """_No description given by bungie._ """
    MUST_NOT = 1
    """_No description given by bungie._ """
    SHOULD = 2
    """_No description given by bungie._ """
    FILTER = 3
    """_No description given by bungie._ """


@custom_define()
class DestinyFireteamFinderUpdateLobbySettingsResponse(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        updated_listing: _No description given by bungie._
        updated_lobby: _No description given by bungie._
    """

    updated_listing: "DestinyFireteamFinderListing" = custom_field()
    updated_lobby: "DestinyFireteamFinderLobbyResponse" = custom_field()


@custom_define()
class DestinyFireteamFinderUpdateLobbySettingsRequest(BaseModel):
    """
    _No description given by bungie._

    None
    Attributes:
        updated_settings: _No description given by bungie._
    """

    updated_settings: "DestinyFireteamFinderLobbySettings" = custom_field()



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from typing import Optional, TYPE_CHECKING

from bungio.models.base import HashObject, ManifestModel, custom_define, custom_field


if TYPE_CHECKING:
    from bungio.models import InterpolationPointFloat
    from bungio.models import DestinyProgressionDefinition


@custom_define()
class DestinyProgressionLevelRequirementDefinition(ManifestModel, HashObject):
    """
    These are pre-constructed collections of data that can be used to determine the Level Requirement for an item given a Progression to be tested (such as the Character's level). For instance, say a character receives a new Auto Rifle, and that Auto Rifle's DestinyInventoryItemDefinition.quality.progressionLevelRequirementHash property is pointing at one of these DestinyProgressionLevelRequirementDefinitions. Let's pretend also that the progressionHash it is pointing at is the Character Level progression. In that situation, the character's level will be used to interpolate a value in the requirementCurve property. The value picked up from that interpolation will be the required level for the item.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        hash: The unique identifier for this entity. Guaranteed to be unique for the type of entity, but not globally. When entities refer to each other in Destiny content, it is this hash that they are referring to.
        index: The index of the entity as it was found in the investment tables.
        progression_hash: The progression whose level should be used to determine the level requirement. Look up the DestinyProgressionDefinition with this hash for more information about the progression in question.
        redacted: If this is true, then there is an entity with this identifier/type combination, but BNet is not yet allowed to show it. Sorry!
        requirement_curve: A curve of level requirements, weighted by the related progressions' level. Interpolate against this curve with the character's progression level to determine what the level requirement of the generated item that is using this data will be.
        manifest_progression_hash: Manifest information for `progression_hash`
    """

    index: int = custom_field()
    progression_hash: int = custom_field()
    redacted: bool = custom_field()
    requirement_curve: list["InterpolationPointFloat"] = custom_field(
        metadata={"type": """list[InterpolationPointFloat]"""}
    )
    manifest_progression_hash: Optional["DestinyProgressionDefinition"] = custom_field(default=None)



from typing import TYPE_CHECKING

from bungio.definitions import DEFAULT_LOGGER
from bungio.models.base import MISSING, custom_define

if TYPE_CHECKING:
    from bungio.client import Client


__all__ = (
    "client",
    "Singleton",
)

client: "Client" = MISSING


_instances: dict = {}


@custom_define()
class Singleton:
    """
    Singleton class used to only create one instance
    """

    def __new__(cls, *args, **kwargs):
        if cls not in _instances:
            _instances[cls] = super().__new__(cls)
            return _instances[cls]
        else:
            found_cls = _instances[cls]
            if client is not MISSING:
                logger = client.logger
            else:
                logger = DEFAULT_LOGGER
            logger.warning(
                f"The `{cls.__name__}` can only be defined once, ignoring your inputs and returning the already defined `{cls.__name__}`"
            )
            return found_cls



from datetime import datetime
from typing import Callable, Coroutine, Optional, Any, Union

from bungio.http.route import Route
from bungio.models.auth import AuthData


class Destiny2RouteHttpRequests:
    request: Callable[..., Coroutine]

    async def get_destiny_manifest(self, auth: Optional[AuthData] = None, *args, **kwargs) -> dict:
        """
        Returns the current version of the manifest as a json object.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(Route(path="/Destiny2/Manifest/", method="GET", auth=auth))

    async def get_destiny_entity_definition(
        self, entity_type: str, hash_identifier: int, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        Returns the static definition of an entity of the given Type and hash identifier. Examine the API Documentation for the Type Names of entities that have their own definitions. Note that the return type will always *inherit from* DestinyDefinition, but the specific type returned will be the requested entity type if it can be found. Please don't use this as a chatty alternative to the Manifest database if you require large sets of data, but for simple and one-off accesses this should be handy.

        Args:
            entity_type: The type of entity for whom you would like results. These correspond to the entity's definition contract name. For instance, if you are looking for items, this property should be 'DestinyInventoryItemDefinition'. PREVIEW: This endpoint is still in beta, and may experience rough edges. The schema is tentatively in final form, but there may be bugs that prevent desirable operation.
            hash_identifier: The hash identifier for the specific Entity you want returned.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(path=f"/Destiny2/Manifest/{entity_type}/{hash_identifier}/", method="GET", auth=auth)
        )

    async def search_destiny_player_by_bungie_name(
        self,
        display_name: str,
        display_name_code: int,
        membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Returns a list of Destiny memberships given a global Bungie Display Name. This method will hide overridden memberships due to cross save.

        Args:
            display_name: _No description given by bungie._
            display_name_code: _No description given by bungie._
            membership_type: A valid non-BungieNet membership type, or All. Indicates which memberships to return. You probably want this set to All.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "displayName": display_name,
            "displayNameCode": display_name_code,
        }

        return await self.request(
            Route(
                path=f"/Destiny2/SearchDestinyPlayerByBungieName/{membership_type}/",
                method="POST",
                data=data,
                auth=auth,
            )
        )

    async def get_linked_profiles(
        self,
        membership_id: int,
        membership_type: int,
        get_all_memberships: Optional[bool] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Returns a summary information about all profiles linked to the requesting membership type/membership ID that have valid Destiny information. The passed-in Membership Type/Membership ID may be a Bungie.Net membership or a Destiny membership. It only returns the minimal amount of data to begin making more substantive requests, but will hopefully serve as a useful alternative to UserServices for people who just care about Destiny data. Note that it will only return linked accounts whose linkages you are allowed to view.

        Args:
            membership_id: The ID of the membership whose linked Destiny accounts you want returned. Make sure your membership ID matches its Membership Type: don't pass us a PSN membership ID and the XBox membership type, it's not going to work!
            membership_type: The type for the membership whose linked Destiny accounts you want returned.
            get_all_memberships: (optional) if set to 'true', all memberships regardless of whether they're obscured by overrides will be returned. Normal privacy restrictions on account linking will still apply no matter what.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Profile/{membership_id}/LinkedProfiles/",
                method="GET",
                get_all_memberships=get_all_memberships,
                auth=auth,
            )
        )

    async def get_profile(
        self,
        destiny_membership_id: int,
        membership_type: int,
        components: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Returns Destiny Profile information for the supplied membership.

        Args:
            destiny_membership_id: Destiny membership ID.
            membership_type: A valid non-BungieNet membership type.
            components: A comma separated list of components to return (as strings or numeric values). See the DestinyComponentType enum for valid components to request. You must request at least one component to receive results.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Profile/{destiny_membership_id}/",
                method="GET",
                components=components,
                auth=auth,
            )
        )

    async def get_character(
        self,
        character_id: int,
        destiny_membership_id: int,
        membership_type: int,
        components: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Returns character information for the supplied character.

        Args:
            character_id: ID of the character.
            destiny_membership_id: Destiny membership ID.
            membership_type: A valid non-BungieNet membership type.
            components: A comma separated list of components to return (as strings or numeric values). See the DestinyComponentType enum for valid components to request. You must request at least one component to receive results.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Profile/{destiny_membership_id}/Character/{character_id}/",
                method="GET",
                components=components,
                auth=auth,
            )
        )

    async def get_clan_weekly_reward_state(
        self, group_id: int, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        Returns information on the weekly clan rewards and if the clan has earned them or not. Note that this will always report rewards as not redeemed.

        Args:
            group_id: A valid group id of clan.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(Route(path=f"/Destiny2/Clan/{group_id}/WeeklyRewardState/", method="GET", auth=auth))

    async def get_clan_banner_source(self, auth: Optional[AuthData] = None, *args, **kwargs) -> dict:
        """
        Returns the dictionary of values for the Clan Banner

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(Route(path="/Destiny2/Clan/ClanBannerDictionary/", method="GET", auth=auth))

    async def get_item(
        self,
        destiny_membership_id: int,
        item_instance_id: int,
        membership_type: int,
        components: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Retrieve the details of an instanced Destiny Item. An instanced Destiny item is one with an ItemInstanceId. Non-instanced items, such as materials, have no useful instance-specific details and thus are not queryable here.

        Args:
            destiny_membership_id: The membership ID of the destiny profile.
            item_instance_id: The Instance ID of the destiny item.
            membership_type: A valid non-BungieNet membership type.
            components: A comma separated list of components to return (as strings or numeric values). See the DestinyComponentType enum for valid components to request. You must request at least one component to receive results.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Profile/{destiny_membership_id}/Item/{item_instance_id}/",
                method="GET",
                components=components,
                auth=auth,
            )
        )

    async def get_vendors(
        self,
        character_id: int,
        destiny_membership_id: int,
        membership_type: int,
        components: Optional[int] = None,
        filter: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Get currently available vendors from the list of vendors that can possibly have rotating inventory. Note that this does not include things like preview vendors and vendors-as-kiosks, neither of whom have rotating/dynamic inventories. Use their definitions as-is for those.

        Args:
            character_id: The Destiny Character ID of the character for whom we're getting vendor info.
            destiny_membership_id: Destiny membership ID of another user. You may be denied.
            membership_type: A valid non-BungieNet membership type.
            components: A comma separated list of components to return (as strings or numeric values). See the DestinyComponentType enum for valid components to request. You must request at least one component to receive results.
            filter: The filter of what vendors and items to return, if any.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Profile/{destiny_membership_id}/Character/{character_id}/Vendors/",
                method="GET",
                components=components,
                filter=filter,
                auth=auth,
            )
        )

    async def get_vendor(
        self,
        character_id: int,
        destiny_membership_id: int,
        membership_type: int,
        vendor_hash: int,
        components: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Get the details of a specific Vendor.

        Args:
            character_id: The Destiny Character ID of the character for whom we're getting vendor info.
            destiny_membership_id: Destiny membership ID of another user. You may be denied.
            membership_type: A valid non-BungieNet membership type.
            vendor_hash: The Hash identifier of the Vendor to be returned.
            components: A comma separated list of components to return (as strings or numeric values). See the DestinyComponentType enum for valid components to request. You must request at least one component to receive results.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Profile/{destiny_membership_id}/Character/{character_id}/Vendors/{vendor_hash}/",
                method="GET",
                components=components,
                auth=auth,
            )
        )

    async def get_public_vendors(
        self, components: Optional[int] = None, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        Get items available from vendors where the vendors have items for sale that are common for everyone. If any portion of the Vendor's available inventory is character or account specific, we will be unable to return their data from this endpoint due to the way that available inventory is computed. As I am often guilty of saying: 'It's a long story...'

        Args:
            components: A comma separated list of components to return (as strings or numeric values). See the DestinyComponentType enum for valid components to request. You must request at least one component to receive results.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(Route(path="/Destiny2/Vendors/", method="GET", components=components, auth=auth))

    async def get_collectible_node_details(
        self,
        character_id: int,
        collectible_presentation_node_hash: int,
        destiny_membership_id: int,
        membership_type: int,
        components: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Given a Presentation Node that has Collectibles as direct descendants, this will return item details about those descendants in the context of the requesting character.

        Args:
            character_id: The Destiny Character ID of the character for whom we're getting collectible detail info.
            collectible_presentation_node_hash: The hash identifier of the Presentation Node for whom we should return collectible details. Details will only be returned for collectibles that are direct descendants of this node.
            destiny_membership_id: Destiny membership ID of another user. You may be denied.
            membership_type: A valid non-BungieNet membership type.
            components: A comma separated list of components to return (as strings or numeric values). See the DestinyComponentType enum for valid components to request. You must request at least one component to receive results.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Profile/{destiny_membership_id}/Character/{character_id}/Collectibles/{collectible_presentation_node_hash}/",
                method="GET",
                components=components,
                auth=auth,
            )
        )

    async def transfer_item(
        self,
        item_reference_hash: int,
        stack_size: int,
        transfer_to_vault: bool,
        item_id: int,
        character_id: int,
        membership_type: Union[Any, int],
        auth: AuthData,
        *args,
        **kwargs,
    ) -> dict:
        """
        Transfer an item to/from your vault. You must have a valid Destiny account. You must also pass BOTH a reference AND an instance ID if it's an instanced item. itshappening.gif

        Warning: Requires Authentication.
            Required oauth2 scopes: MoveEquipDestinyItems

        Args:
            item_reference_hash: _No description given by bungie._
            stack_size: _No description given by bungie._
            transfer_to_vault: _No description given by bungie._
            item_id: The instance ID of the item for this action request.
            character_id: _No description given by bungie._
            membership_type: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "itemReferenceHash": item_reference_hash,
            "stackSize": stack_size,
            "transferToVault": transfer_to_vault,
            "itemId": item_id,
            "characterId": character_id,
            "membershipType": membership_type,
        }

        return await self.request(
            Route(path="/Destiny2/Actions/Items/TransferItem/", method="POST", data=data, auth=auth)
        )

    async def pull_from_postmaster(
        self,
        item_reference_hash: int,
        stack_size: int,
        item_id: int,
        character_id: int,
        membership_type: Union[Any, int],
        auth: AuthData,
        *args,
        **kwargs,
    ) -> dict:
        """
        Extract an item from the Postmaster, with whatever implications that may entail. You must have a valid Destiny account. You must also pass BOTH a reference AND an instance ID if it's an instanced item.

        Warning: Requires Authentication.
            Required oauth2 scopes: MoveEquipDestinyItems

        Args:
            item_reference_hash: _No description given by bungie._
            stack_size: _No description given by bungie._
            item_id: The instance ID of the item for this action request.
            character_id: _No description given by bungie._
            membership_type: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "itemReferenceHash": item_reference_hash,
            "stackSize": stack_size,
            "itemId": item_id,
            "characterId": character_id,
            "membershipType": membership_type,
        }

        return await self.request(
            Route(path="/Destiny2/Actions/Items/PullFromPostmaster/", method="POST", data=data, auth=auth)
        )

    async def equip_item(
        self, item_id: int, character_id: int, membership_type: Union[Any, int], auth: AuthData, *args, **kwargs
    ) -> dict:
        """
        Equip an item. You must have a valid Destiny Account, and either be in a social space, in orbit, or offline.

        Warning: Requires Authentication.
            Required oauth2 scopes: MoveEquipDestinyItems

        Args:
            item_id: The instance ID of the item for this action request.
            character_id: _No description given by bungie._
            membership_type: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "itemId": item_id,
            "characterId": character_id,
            "membershipType": membership_type,
        }

        return await self.request(Route(path="/Destiny2/Actions/Items/EquipItem/", method="POST", data=data, auth=auth))

    async def equip_items(
        self, item_ids: list[int], character_id: int, membership_type: Union[Any, int], auth: AuthData, *args, **kwargs
    ) -> dict:
        """
        Equip a list of items by itemInstanceIds. You must have a valid Destiny Account, and either be in a social space, in orbit, or offline. Any items not found on your character will be ignored.

        Warning: Requires Authentication.
            Required oauth2 scopes: MoveEquipDestinyItems

        Args:
            item_ids: _No description given by bungie._
            character_id: _No description given by bungie._
            membership_type: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "itemIds": item_ids,
            "characterId": character_id,
            "membershipType": membership_type,
        }

        return await self.request(
            Route(path="/Destiny2/Actions/Items/EquipItems/", method="POST", data=data, auth=auth)
        )

    async def equip_loadout(
        self, loadout_index: int, character_id: int, membership_type: Union[Any, int], auth: AuthData, *args, **kwargs
    ) -> dict:
        """
        Equip a loadout. You must have a valid Destiny Account, and either be in a social space, in orbit, or offline.

        Warning: Requires Authentication.
            Required oauth2 scopes: MoveEquipDestinyItems

        Args:
            loadout_index: The index of the loadout for this action request.
            character_id: _No description given by bungie._
            membership_type: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "loadoutIndex": loadout_index,
            "characterId": character_id,
            "membershipType": membership_type,
        }

        return await self.request(
            Route(path="/Destiny2/Actions/Loadouts/EquipLoadout/", method="POST", data=data, auth=auth)
        )

    async def snapshot_loadout(
        self,
        color_hash: int,
        icon_hash: int,
        name_hash: int,
        loadout_index: int,
        character_id: int,
        membership_type: Union[Any, int],
        auth: AuthData,
        *args,
        **kwargs,
    ) -> dict:
        """
        Snapshot a loadout with the currently equipped items.

        Warning: Requires Authentication.
            Required oauth2 scopes: MoveEquipDestinyItems

        Args:
            color_hash: _No description given by bungie._
            icon_hash: _No description given by bungie._
            name_hash: _No description given by bungie._
            loadout_index: The index of the loadout for this action request.
            character_id: _No description given by bungie._
            membership_type: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "colorHash": color_hash,
            "iconHash": icon_hash,
            "nameHash": name_hash,
            "loadoutIndex": loadout_index,
            "characterId": character_id,
            "membershipType": membership_type,
        }

        return await self.request(
            Route(path="/Destiny2/Actions/Loadouts/SnapshotLoadout/", method="POST", data=data, auth=auth)
        )

    async def update_loadout_identifiers(
        self,
        color_hash: int,
        icon_hash: int,
        name_hash: int,
        loadout_index: int,
        character_id: int,
        membership_type: Union[Any, int],
        auth: AuthData,
        *args,
        **kwargs,
    ) -> dict:
        """
        Update the color, icon, and name of a loadout.

        Warning: Requires Authentication.
            Required oauth2 scopes: MoveEquipDestinyItems

        Args:
            color_hash: _No description given by bungie._
            icon_hash: _No description given by bungie._
            name_hash: _No description given by bungie._
            loadout_index: The index of the loadout for this action request.
            character_id: _No description given by bungie._
            membership_type: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "colorHash": color_hash,
            "iconHash": icon_hash,
            "nameHash": name_hash,
            "loadoutIndex": loadout_index,
            "characterId": character_id,
            "membershipType": membership_type,
        }

        return await self.request(
            Route(path="/Destiny2/Actions/Loadouts/UpdateLoadoutIdentifiers/", method="POST", data=data, auth=auth)
        )

    async def clear_loadout(
        self, loadout_index: int, character_id: int, membership_type: Union[Any, int], auth: AuthData, *args, **kwargs
    ) -> dict:
        """
        Clear the identifiers and items of a loadout.

        Warning: Requires Authentication.
            Required oauth2 scopes: MoveEquipDestinyItems

        Args:
            loadout_index: The index of the loadout for this action request.
            character_id: _No description given by bungie._
            membership_type: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "loadoutIndex": loadout_index,
            "characterId": character_id,
            "membershipType": membership_type,
        }

        return await self.request(
            Route(path="/Destiny2/Actions/Loadouts/ClearLoadout/", method="POST", data=data, auth=auth)
        )

    async def set_item_lock_state(
        self,
        state: bool,
        item_id: int,
        character_id: int,
        membership_type: Union[Any, int],
        auth: AuthData,
        *args,
        **kwargs,
    ) -> dict:
        """
        Set the Lock State for an instanced item. You must have a valid Destiny Account.

        Warning: Requires Authentication.
            Required oauth2 scopes: MoveEquipDestinyItems

        Args:
            state: _No description given by bungie._
            item_id: The instance ID of the item for this action request.
            character_id: _No description given by bungie._
            membership_type: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "state": state,
            "itemId": item_id,
            "characterId": character_id,
            "membershipType": membership_type,
        }

        return await self.request(
            Route(path="/Destiny2/Actions/Items/SetLockState/", method="POST", data=data, auth=auth)
        )

    async def set_quest_tracked_state(
        self,
        state: bool,
        item_id: int,
        character_id: int,
        membership_type: Union[Any, int],
        auth: AuthData,
        *args,
        **kwargs,
    ) -> dict:
        """
        Set the Tracking State for an instanced item, if that item is a Quest or Bounty. You must have a valid Destiny Account. Yeah, it's an item.

        Warning: Requires Authentication.
            Required oauth2 scopes: MoveEquipDestinyItems

        Args:
            state: _No description given by bungie._
            item_id: The instance ID of the item for this action request.
            character_id: _No description given by bungie._
            membership_type: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "state": state,
            "itemId": item_id,
            "characterId": character_id,
            "membershipType": membership_type,
        }

        return await self.request(
            Route(path="/Destiny2/Actions/Items/SetTrackedState/", method="POST", data=data, auth=auth)
        )

    async def insert_socket_plug(
        self,
        action_token: str,
        item_instance_id: int,
        plug: Any,
        character_id: int,
        membership_type: Union[Any, int],
        auth: AuthData,
        *args,
        **kwargs,
    ) -> dict:
        """
        Insert a plug into a socketed item. I know how it sounds, but I assure you it's much more G-rated than you might be guessing. We haven't decided yet whether this will be able to insert plugs that have side effects, but if we do it will require special scope permission for an application attempting to do so. You must have a valid Destiny Account, and either be in a social space, in orbit, or offline. Request must include proof of permission for 'InsertPlugs' from the account owner.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdvancedWriteActions

        Args:
            action_token: Action token provided by the AwaGetActionToken API call.
            item_instance_id: The instance ID of the item having a plug inserted. Only instanced items can have sockets.
            plug: The plugs being inserted.
            character_id: _No description given by bungie._
            membership_type: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "actionToken": action_token,
            "itemInstanceId": item_instance_id,
            "plug": plug,
            "characterId": character_id,
            "membershipType": membership_type,
        }

        return await self.request(
            Route(path="/Destiny2/Actions/Items/InsertSocketPlug/", method="POST", data=data, auth=auth)
        )

    async def insert_socket_plug_free(
        self,
        plug: Any,
        item_id: int,
        character_id: int,
        membership_type: Union[Any, int],
        auth: AuthData,
        *args,
        **kwargs,
    ) -> dict:
        """
        Insert a 'free' plug into an item's socket. This does not require 'Advanced Write Action' authorization and is available to 3rd-party apps, but will only work on 'free and reversible' socket actions (Perks, Armor Mods, Shaders, Ornaments, etc.). You must have a valid Destiny Account, and the character must either be in a social space, in orbit, or offline.

        Warning: Requires Authentication.
            Required oauth2 scopes: MoveEquipDestinyItems

        Args:
            plug: The plugs being inserted.
            item_id: The instance ID of the item for this action request.
            character_id: _No description given by bungie._
            membership_type: _No description given by bungie._
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "plug": plug,
            "itemId": item_id,
            "characterId": character_id,
            "membershipType": membership_type,
        }

        return await self.request(
            Route(path="/Destiny2/Actions/Items/InsertSocketPlugFree/", method="POST", data=data, auth=auth)
        )

    async def get_post_game_carnage_report(
        self, activity_id: int, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        Gets the available post game carnage report for the activity ID.

        Args:
            activity_id: The ID of the activity whose PGCR is requested.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(path=f"/Destiny2/Stats/PostGameCarnageReport/{activity_id}/", method="GET", auth=auth)
        )

    async def report_offensive_post_game_carnage_report_player(
        self,
        reason_category_hashes: list[int],
        reason_hashes: list[int],
        offending_character_id: int,
        activity_id: int,
        auth: AuthData,
        *args,
        **kwargs,
    ) -> dict:
        """
        Report a player that you met in an activity that was engaging in ToS-violating activities. Both you and the offending player must have played in the activityId passed in. Please use this judiciously and only when you have strong suspicions of violation, pretty please.

        Warning: Requires Authentication.
            Required oauth2 scopes: BnetWrite

        Args:
            reason_category_hashes: So you've decided to report someone instead of cursing them and their descendants. Well, okay then. This is the category or categorie(s) of infractions for which you are reporting the user. These are hash identifiers that map to DestinyReportReasonCategoryDefinition entries.
            reason_hashes: If applicable, provide a more specific reason(s) within the general category of problems provided by the reasonHash. This is also an identifier for a reason. All reasonHashes provided must be children of at least one the reasonCategoryHashes provided.
            offending_character_id: Within the PGCR provided when calling the Reporting endpoint, this should be the character ID of the user that you thought was violating terms of use. They must exist in the PGCR provided.
            activity_id: The ID of the activity where you ran into the brigand that you're reporting.
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "reasonCategoryHashes": reason_category_hashes,
            "reasonHashes": reason_hashes,
            "offendingCharacterId": offending_character_id,
        }

        return await self.request(
            Route(
                path=f"/Destiny2/Stats/PostGameCarnageReport/{activity_id}/Report/", method="POST", data=data, auth=auth
            )
        )

    async def get_historical_stats_definition(self, auth: Optional[AuthData] = None, *args, **kwargs) -> dict:
        """
        Gets historical stats definitions.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(Route(path="/Destiny2/Stats/Definition/", method="GET", auth=auth))

    async def get_clan_leaderboards(
        self,
        group_id: int,
        maxtop: Optional[int] = None,
        modes: Optional[str] = None,
        statid: Optional[str] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Gets leaderboards with the signed in user's friends and the supplied destinyMembershipId as the focus. PREVIEW: This endpoint is still in beta, and may experience rough edges. The schema is in final form, but there may be bugs that prevent desirable operation.

        Args:
            group_id: Group ID of the clan whose leaderboards you wish to fetch.
            maxtop: Maximum number of top players to return. Use a large number to get entire leaderboard.
            modes: List of game modes for which to get leaderboards. See the documentation for DestinyActivityModeType for valid values, and pass in string representation, comma delimited.
            statid: ID of stat to return rather than returning all Leaderboard stats.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/Stats/Leaderboards/Clans/{group_id}/",
                method="GET",
                maxtop=maxtop,
                modes=modes,
                statid=statid,
                auth=auth,
            )
        )

    async def get_clan_aggregate_stats(
        self, group_id: int, modes: Optional[str] = None, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        Gets aggregated stats for a clan using the same categories as the clan leaderboards. PREVIEW: This endpoint is still in beta, and may experience rough edges. The schema is in final form, but there may be bugs that prevent desirable operation.

        Args:
            group_id: Group ID of the clan whose leaderboards you wish to fetch.
            modes: List of game modes for which to get leaderboards. See the documentation for DestinyActivityModeType for valid values, and pass in string representation, comma delimited.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(path=f"/Destiny2/Stats/AggregateClanStats/{group_id}/", method="GET", modes=modes, auth=auth)
        )

    async def get_leaderboards(
        self,
        destiny_membership_id: int,
        membership_type: int,
        maxtop: Optional[int] = None,
        modes: Optional[str] = None,
        statid: Optional[str] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Gets leaderboards with the signed in user's friends and the supplied destinyMembershipId as the focus. PREVIEW: This endpoint has not yet been implemented. It is being returned for a preview of future functionality, and for public comment/suggestion/preparation.

        Args:
            destiny_membership_id: The Destiny membershipId of the user to retrieve.
            membership_type: A valid non-BungieNet membership type.
            maxtop: Maximum number of top players to return. Use a large number to get entire leaderboard.
            modes: List of game modes for which to get leaderboards. See the documentation for DestinyActivityModeType for valid values, and pass in string representation, comma delimited.
            statid: ID of stat to return rather than returning all Leaderboard stats.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Account/{destiny_membership_id}/Stats/Leaderboards/",
                method="GET",
                maxtop=maxtop,
                modes=modes,
                statid=statid,
                auth=auth,
            )
        )

    async def get_leaderboards_for_character(
        self,
        character_id: int,
        destiny_membership_id: int,
        membership_type: int,
        maxtop: Optional[int] = None,
        modes: Optional[str] = None,
        statid: Optional[str] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Gets leaderboards with the signed in user's friends and the supplied destinyMembershipId as the focus. PREVIEW: This endpoint is still in beta, and may experience rough edges. The schema is in final form, but there may be bugs that prevent desirable operation.

        Args:
            character_id: The specific character to build the leaderboard around for the provided Destiny Membership.
            destiny_membership_id: The Destiny membershipId of the user to retrieve.
            membership_type: A valid non-BungieNet membership type.
            maxtop: Maximum number of top players to return. Use a large number to get entire leaderboard.
            modes: List of game modes for which to get leaderboards. See the documentation for DestinyActivityModeType for valid values, and pass in string representation, comma delimited.
            statid: ID of stat to return rather than returning all Leaderboard stats.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/Stats/Leaderboards/{membership_type}/{destiny_membership_id}/{character_id}/",
                method="GET",
                maxtop=maxtop,
                modes=modes,
                statid=statid,
                auth=auth,
            )
        )

    async def search_destiny_entities(
        self, search_term: str, type: str, page: Optional[int] = None, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        Gets a page list of Destiny items.

        Args:
            search_term: The string to use when searching for Destiny entities.
            type: The type of entity for whom you would like results. These correspond to the entity's definition contract name. For instance, if you are looking for items, this property should be 'DestinyInventoryItemDefinition'.
            page: Page number to return, starting with 0.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(path=f"/Destiny2/Armory/Search/{type}/{search_term}/", method="GET", page=page, auth=auth)
        )

    async def get_historical_stats(
        self,
        character_id: int,
        destiny_membership_id: int,
        membership_type: int,
        dayend: Optional[datetime] = None,
        daystart: Optional[datetime] = None,
        groups: Optional[int] = None,
        modes: Optional[int] = None,
        period_type: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Gets historical stats for indicated character.

        Args:
            character_id: The id of the character to retrieve. You can omit this character ID or set it to 0 to get aggregate stats across all characters.
            destiny_membership_id: The Destiny membershipId of the user to retrieve.
            membership_type: A valid non-BungieNet membership type.
            dayend: Last day to return when daily stats are requested. Use the format YYYY-MM-DD. Currently, we cannot allow more than 31 days of daily data to be requested in a single request.
            daystart: First day to return when daily stats are requested. Use the format YYYY-MM-DD. Currently, we cannot allow more than 31 days of daily data to be requested in a single request.
            groups: Group of stats to include, otherwise only general stats are returned. Comma separated list is allowed. Values: General, Weapons, Medals
            modes: Game modes to return. See the documentation for DestinyActivityModeType for valid values, and pass in string representation, comma delimited.
            period_type: Indicates a specific period type to return. Optional. May be: Daily, AllTime, or Activity
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Account/{destiny_membership_id}/Character/{character_id}/Stats/",
                method="GET",
                dayend=dayend,
                daystart=daystart,
                groups=groups,
                modes=modes,
                period_type=period_type,
                auth=auth,
            )
        )

    async def get_historical_stats_for_account(
        self,
        destiny_membership_id: int,
        membership_type: int,
        groups: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Gets aggregate historical stats organized around each character for a given account.

        Args:
            destiny_membership_id: The Destiny membershipId of the user to retrieve.
            membership_type: A valid non-BungieNet membership type.
            groups: Groups of stats to include, otherwise only general stats are returned. Comma separated list is allowed. Values: General, Weapons, Medals.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Account/{destiny_membership_id}/Stats/",
                method="GET",
                groups=groups,
                auth=auth,
            )
        )

    async def get_activity_history(
        self,
        character_id: int,
        destiny_membership_id: int,
        membership_type: int,
        count: Optional[int] = None,
        mode: Optional[int] = None,
        page: Optional[int] = None,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Gets activity history stats for indicated character.

        Args:
            character_id: The id of the character to retrieve.
            destiny_membership_id: The Destiny membershipId of the user to retrieve.
            membership_type: A valid non-BungieNet membership type.
            count: Number of rows to return
            mode: A filter for the activity mode to be returned. None returns all activities. See the documentation for DestinyActivityModeType for valid values, and pass in string representation.
            page: Page number to return, starting with 0.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Account/{destiny_membership_id}/Character/{character_id}/Stats/Activities/",
                method="GET",
                count=count,
                mode=mode,
                page=page,
                auth=auth,
            )
        )

    async def get_unique_weapon_history(
        self,
        character_id: int,
        destiny_membership_id: int,
        membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Gets details about unique weapon usage, including all exotic weapons.

        Args:
            character_id: The id of the character to retrieve.
            destiny_membership_id: The Destiny membershipId of the user to retrieve.
            membership_type: A valid non-BungieNet membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Account/{destiny_membership_id}/Character/{character_id}/Stats/UniqueWeapons/",
                method="GET",
                auth=auth,
            )
        )

    async def get_destiny_aggregate_activity_stats(
        self,
        character_id: int,
        destiny_membership_id: int,
        membership_type: int,
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Gets all activities the character has participated in together with aggregate statistics for those activities.

        Args:
            character_id: The specific character whose activities should be returned.
            destiny_membership_id: The Destiny membershipId of the user to retrieve.
            membership_type: A valid non-BungieNet membership type.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(
                path=f"/Destiny2/{membership_type}/Account/{destiny_membership_id}/Character/{character_id}/Stats/AggregateActivityStats/",
                method="GET",
                auth=auth,
            )
        )

    async def get_public_milestone_content(
        self, milestone_hash: int, auth: Optional[AuthData] = None, *args, **kwargs
    ) -> dict:
        """
        Gets custom localized content for the milestone of the given hash, if it exists.

        Args:
            milestone_hash: The identifier for the milestone to be returned.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(path=f"/Destiny2/Milestones/{milestone_hash}/Content/", method="GET", auth=auth)
        )

    async def get_public_milestones(self, auth: Optional[AuthData] = None, *args, **kwargs) -> dict:
        """
        Gets public information about currently available Milestones.

        Args:
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(Route(path="/Destiny2/Milestones/", method="GET", auth=auth))

    async def awa_initialize_request(
        self,
        type: Union[Any, int],
        affected_item_id: int,
        membership_type: Union[Any, int],
        character_id: int,
        auth: AuthData,
        *args,
        **kwargs,
    ) -> dict:
        """
        Initialize a request to perform an advanced write action.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdvancedWriteActions

        Args:
            type: Type of advanced write action.
            affected_item_id: Item instance ID the action shall be applied to. This is optional for all but a new AwaType values. Rule of thumb is to provide the item instance ID if one is available.
            membership_type: Destiny membership type of the account to modify.
            character_id: Destiny character ID, if applicable, that will be affected by the action.
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "type": type,
            "affectedItemId": affected_item_id,
            "membershipType": membership_type,
            "characterId": character_id,
        }

        return await self.request(Route(path="/Destiny2/Awa/Initialize/", method="POST", data=data, auth=auth))

    async def awa_provide_authorization_result(
        self,
        selection: Union[Any, int],
        correlation_id: str,
        nonce: list[int],
        auth: Optional[AuthData] = None,
        *args,
        **kwargs,
    ) -> dict:
        """
        Provide the result of the user interaction. Called by the Bungie Destiny App to approve or reject a request.

        Args:
            selection: Indication of the selection the user has made (Approving or rejecting the action)
            correlation_id: Correlation ID of the request
            nonce: Secret nonce received via the PUSH notification.
            auth: Authentication information. Required when users with a private profile are queried, or when Bungie feels like it

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        data = {
            "selection": selection,
            "correlationId": correlation_id,
            "nonce": nonce,
        }

        return await self.request(
            Route(path="/Destiny2/Awa/AwaProvideAuthorizationResult/", method="POST", data=data, auth=auth)
        )

    async def awa_get_action_token(self, correlation_id: str, auth: AuthData, *args, **kwargs) -> dict:
        """
        Returns the action token if user approves the request.

        Warning: Requires Authentication.
            Required oauth2 scopes: AdvancedWriteActions

        Args:
            correlation_id: The identifier for the advanced write action request.
            auth: Authentication information.

        Raises:
            NotFound: 404 request
            BadRequest: 400 request
            InvalidAuthentication: If authentication is invalid
            TimeoutException: If no connection could be made
            BungieDead: Servers are down
            AuthenticationTooSlow: The authentication key has expired
            BungieException: Relaying the bungie error

        Returns:
            The json response
        """

        return await self.request(
            Route(path=f"/Destiny2/Awa/GetActionToken/{correlation_id}/", method="GET", auth=auth)
        )



# DO NOT CHANGE ANY CODE BELOW
# This file is generated automatically by `generate_api_schema.py` and will be overwritten
# Instead, change functions / models by subclassing them in the `./overwrites/` folder. They will be used instead.

from datetime import datetime
from typing import Optional, TYPE_CHECKING

from bungio.models.base import BaseModel, custom_define, custom_field


if TYPE_CHECKING:
    from bungio.models import DestinyActivityModeDefinition
    from bungio.models import DestinyQuestStatus
    from bungio.models import DestinyInventoryItemDefinition
    from bungio.models import DestinyMilestoneDefinition
    from bungio.models import DestinyVendorDefinition
    from bungio.models import DestinyChallengeStatus
    from bungio.models import DestinyObjectiveDefinition
    from bungio.models import DestinyActivityDefinition


@custom_define()
class DestinyMilestone(BaseModel):
    """
    Represents a runtime instance of a user's milestone status. Live Milestone data should be combined with DestinyMilestoneDefinition data to show the user a picture of what is available for them to do in the game, and their status in regards to said "things to do." Consider it a big, wonky to-do list, or Advisors 3.0 for those who remember the Destiny 1 API.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activities: The currently active Activities in this milestone, when the Milestone is driven by Challenges. Not all Milestones have Challenges, but when they do this will indicate the Activities and Challenges under those Activities related to this Milestone.
        available_quests: Indicates what quests are available for this Milestone. Usually this will be only a single Quest, but some quests have multiple available that you can choose from at any given time. All possible quests for a milestone can be found in the DestinyMilestoneDefinition, but they must be combined with this Live data to determine which one(s) are actually active right now. It is possible for Milestones to not have any quests.
        end_date: If known, this is the date when the event will next end or repeat. It will only be populated for events with fixed and repeating start and end dates.
        milestone_hash: The unique identifier for the Milestone. Use it to look up the DestinyMilestoneDefinition, so you can combine the other data in this contract with static definition data.
        order: Used for ordering milestones in a display to match how we order them in BNet. May pull from static data, or possibly in the future from dynamic information.
        rewards: If the entity to which this component is attached has known active Rewards for the player, this will detail information about those rewards, keyed by the RewardEntry Hash. (See DestinyMilestoneDefinition for more information about Reward Entries) Note that these rewards are not for the Quests related to the Milestone. Think of these as "overview/checklist" rewards that may be provided for Milestones that may provide rewards for performing a variety of tasks that aren't under a specific Quest.
        start_date: If known, this is the date when the event last began or refreshed. It will only be populated for events with fixed and repeating start and end dates.
        values: Milestones may have arbitrary key/value pairs associated with them, for data that users will want to know about but that doesn't fit neatly into any of the common components such as Quests. A good example of this would be - if this existed in Destiny 1 - the number of wins you currently have on your Trials of Osiris ticket. Looking in the DestinyMilestoneDefinition, you can use the string identifier of this dictionary to look up more info about the value, including localized string content for displaying the value. The value in the dictionary is the floating point number. The definition will tell you how to format this number.
        vendor_hashes: A milestone may have one or more active vendors that are "related" to it (that provide rewards, or that are the initiators of the Milestone). I already regret this, even as I'm typing it. [I told you I'd regret this] You see, sometimes a milestone may be directly correlated with a set of vendors that provide varying tiers of rewards. The player may not be able to interact with one or more of those vendors. This will return the hashes of the Vendors that the player *can* interact with, allowing you to show their current inventory as rewards or related items to the Milestone or its activities. Before we even use it, it's already deprecated! How much of a bummer is that? We need more data.
        vendors: Replaces vendorHashes, which I knew was going to be trouble the day it walked in the door. This will return not only what Vendors are active and relevant to the activity (in an implied order that you can choose to ignore), but also other data - for example, if the Vendor is featuring a specific item relevant to this event that you should show with them.
        manifest_milestone_hash: Manifest information for `milestone_hash`
    """

    activities: list["DestinyMilestoneChallengeActivity"] = custom_field(
        metadata={"type": """list[DestinyMilestoneChallengeActivity]"""}
    )
    available_quests: list["DestinyMilestoneQuest"] = custom_field(metadata={"type": """list[DestinyMilestoneQuest]"""})
    end_date: datetime = custom_field()
    milestone_hash: int = custom_field()
    order: int = custom_field()
    rewards: list["DestinyMilestoneRewardCategory"] = custom_field(
        metadata={"type": """list[DestinyMilestoneRewardCategory]"""}
    )
    start_date: datetime = custom_field()
    values: dict[str, float] = custom_field(metadata={"type": """dict[str, float]"""})
    vendor_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    vendors: list["DestinyMilestoneVendor"] = custom_field(metadata={"type": """list[DestinyMilestoneVendor]"""})
    manifest_milestone_hash: Optional["DestinyMilestoneDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneQuest(BaseModel):
    """
    If a Milestone has one or more Quests, this will contain the live information for the character's status with one of those quests.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity: *IF* the Milestone has an active Activity that can give you greater details about what you need to do, it will be returned here. Remember to associate this with the DestinyMilestoneDefinition's activities to get details about the activity, including what specific quest it is related to if you have multiple quests to choose from.
        challenges: The activities referred to by this quest can have many associated challenges. They are all contained here, with activityHashes so that you can associate them with the specific activity variants in which they can be found. In retrospect, I probably should have put these under the specific Activity Variants, but it's too late to change it now. Theoretically, a quest without Activities can still have Challenges, which is why this is on a higher level than activity/variants, but it probably should have been in both places. That may come as a later revision.
        quest_item_hash: Quests are defined as Items in content. As such, this is the hash identifier of the DestinyInventoryItemDefinition that represents this quest. It will have pointers to all of the steps in the quest, and display information for the quest (title, description, icon etc) Individual steps will be referred to in the Quest item's DestinyInventoryItemDefinition.setData property, and themselves are Items with their own renderable data.
        status: The current status of the quest for the character making the request.
        manifest_quest_item_hash: Manifest information for `quest_item_hash`
    """

    activity: "DestinyMilestoneActivity" = custom_field()
    challenges: list["DestinyChallengeStatus"] = custom_field(metadata={"type": """list[DestinyChallengeStatus]"""})
    quest_item_hash: int = custom_field()
    status: "DestinyQuestStatus" = custom_field()
    manifest_quest_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneActivity(BaseModel):
    """
    Sometimes, we know the specific activity that the Milestone wants you to play. This entity provides additional information about that Activity and all of its variants. (sometimes there's only one variant, but I think you get the point)

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_hash: The hash of an arbitrarily chosen variant of this activity. We'll go ahead and call that the "canonical" activity, because if you're using this value you should only use it for properties that are common across the variants: things like the name of the activity, it's location, etc... Use this hash to look up the DestinyActivityDefinition of this activity for rendering data.
        activity_mode_hash: The hash identifier of the most specific Activity Mode under which this activity is played. This is useful for situations where the activity in question is - for instance - a PVP map, but it's not clear what mode the PVP map is being played under. If it's a playlist, this will be less specific: but hopefully useful in some way.
        activity_mode_type: The enumeration equivalent of the most specific Activity Mode under which this activity is played.
        modifier_hashes: If the activity has modifiers, this will be the list of modifiers that all variants have in common. Perform lookups against DestinyActivityModifierDefinition which defines the modifier being applied to get at the modifier data. Note that, in the DestiyActivityDefinition, you will see many more modifiers than this being referred to: those are all *possible* modifiers for the activity, not the active ones. Use only the active ones to match what's really live.
        variants: If you want more than just name/location/etc... you're going to have to dig into and show the variants of the conceptual activity. These will differ in seemingly arbitrary ways, like difficulty level and modifiers applied. Show it in whatever way tickles your fancy.
        manifest_activity_hash: Manifest information for `activity_hash`
        manifest_activity_mode_hash: Manifest information for `activity_mode_hash`
    """

    activity_hash: int = custom_field()
    activity_mode_hash: int = custom_field()
    activity_mode_type: int = custom_field()
    modifier_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    variants: list["DestinyMilestoneActivityVariant"] = custom_field(
        metadata={"type": """list[DestinyMilestoneActivityVariant]"""}
    )
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)
    manifest_activity_mode_hash: Optional["DestinyActivityModeDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneActivityVariant(BaseModel):
    """
    Represents custom data that we know about an individual variant of an activity.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_hash: The hash for the specific variant of the activity related to this milestone. You can pull more detailed static info from the DestinyActivityDefinition, such as difficulty level.
        activity_mode_hash: The hash identifier of the most specific Activity Mode under which this activity is played. This is useful for situations where the activity in question is - for instance - a PVP map, but it's not clear what mode the PVP map is being played under. If it's a playlist, this will be less specific: but hopefully useful in some way.
        activity_mode_type: The enumeration equivalent of the most specific Activity Mode under which this activity is played.
        completion_status: An OPTIONAL component: if it makes sense to talk about this activity variant in terms of whether or not it has been completed or what progress you have made in it, this will be returned. Otherwise, this will be NULL.
        manifest_activity_hash: Manifest information for `activity_hash`
        manifest_activity_mode_hash: Manifest information for `activity_mode_hash`
    """

    activity_hash: int = custom_field()
    activity_mode_hash: int = custom_field()
    activity_mode_type: int = custom_field()
    completion_status: "DestinyMilestoneActivityCompletionStatus" = custom_field()
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)
    manifest_activity_mode_hash: Optional["DestinyActivityModeDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneActivityCompletionStatus(BaseModel):
    """
    Represents this player's personal completion status for the Activity under a Milestone, if the activity has trackable completion and progress information. (most activities won't, or the concept won't apply. For instance, it makes sense to talk about a tier of a raid as being Completed or having progress, but it doesn't make sense to talk about a Crucible Playlist in those terms.

    None
    Attributes:
        completed: If the activity has been "completed", that information will be returned here.
        phases: If the Activity has discrete "phases" that we can track, that info will be here. Otherwise, this value will be NULL. Note that this is a list and not a dictionary: the order implies the ascending order of phases or progression in this activity.
    """

    completed: bool = custom_field()
    phases: list["DestinyMilestoneActivityPhase"] = custom_field(
        metadata={"type": """list[DestinyMilestoneActivityPhase]"""}
    )


@custom_define()
class DestinyMilestoneActivityPhase(BaseModel):
    """
    Represents whatever information we can return about an explicit phase in an activity. In the future, I hope we'll have more than just "guh, you done gone and did something," but for the forseeable future that's all we've got. I'm making it more than just a list of booleans out of that overly-optimistic hope.

    None
    Attributes:
        complete: Indicates if the phase has been completed.
        phase_hash: In DestinyActivityDefinition, if the activity has phases, there will be a set of phases defined in the "insertionPoints" property. This is the hash that maps to that phase.
    """

    complete: bool = custom_field()
    phase_hash: int = custom_field()


@custom_define()
class DestinyMilestoneChallengeActivity(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_hash: _No description given by bungie._
        boolean_activity_options: The set of activity options for this activity, keyed by an identifier that's unique for this activity (not guaranteed to be unique between or across all activities, though should be unique for every *variant* of a given *conceptual* activity: for instance, the original D2 Raid has many variant DestinyActivityDefinitions. While other activities could potentially have the same option hashes, for any given D2 base Raid variant the hash will be unique). As a concrete example of this data, the hashes you get for Raids will correspond to the currently active "Challenge Mode". We don't have any human readable information for these, but saavy 3rd party app users could manually associate the key (a hash identifier for the "option" that is enabled/disabled) and the value (whether it's enabled or disabled presently) On our side, we don't necessarily even know what these are used for (the game designers know, but we don't), and we have no human readable data for them. In order to use them, you will have to do some experimentation.
        challenges: _No description given by bungie._
        loadout_requirement_index: If returned, this is the index into the DestinyActivityDefinition's "loadouts" property, indicating the currently active loadout requirements.
        modifier_hashes: If the activity has modifiers, this will be the list of modifiers that all variants have in common. Perform lookups against DestinyActivityModifierDefinition which defines the modifier being applied to get at the modifier data. Note that, in the DestiyActivityDefinition, you will see many more modifiers than this being referred to: those are all *possible* modifiers for the activity, not the active ones. Use only the active ones to match what's really live.
        phases: If the Activity has discrete "phases" that we can track, that info will be here. Otherwise, this value will be NULL. Note that this is a list and not a dictionary: the order implies the ascending order of phases or progression in this activity.
        manifest_activity_hash: Manifest information for `activity_hash`
    """

    activity_hash: int = custom_field()
    boolean_activity_options: dict[int, bool] = custom_field(metadata={"type": """dict[int, bool]"""})
    challenges: list["DestinyChallengeStatus"] = custom_field(metadata={"type": """list[DestinyChallengeStatus]"""})
    loadout_requirement_index: int = custom_field()
    modifier_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    phases: list["DestinyMilestoneActivityPhase"] = custom_field(
        metadata={"type": """list[DestinyMilestoneActivityPhase]"""}
    )
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneVendor(BaseModel):
    """
    If a Milestone has one or more Vendors that are relevant to it, this will contain information about that vendor that you can choose to show.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        preview_item_hash: If this vendor is featuring a specific item for this event, this will be the hash identifier of that item. I'm taking bets now on how long we go before this needs to be a list or some other, more complex representation instead and I deprecate this too. I'm going to go with 5 months. Calling it now, 2017-09-14 at 9:46pm PST.
        vendor_hash: The hash identifier of the Vendor related to this Milestone. You can show useful things from this, such as thier Faction icon or whatever you might care about.
        manifest_preview_item_hash: Manifest information for `preview_item_hash`
        manifest_vendor_hash: Manifest information for `vendor_hash`
    """

    preview_item_hash: int = custom_field()
    vendor_hash: int = custom_field()
    manifest_preview_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)
    manifest_vendor_hash: Optional["DestinyVendorDefinition"] = custom_field(default=None)


@custom_define()
class DestinyMilestoneRewardCategory(BaseModel):
    """
    Represents a category of "summary" rewards that can be earned for the Milestone regardless of specific quest rewards that can be earned.

    None
    Attributes:
        entries: The individual reward entries for this category, and their status.
        reward_category_hash: Look up the relevant DestinyMilestoneDefinition, and then use rewardCategoryHash to look up the category info in DestinyMilestoneDefinition.rewards.
    """

    entries: list["DestinyMilestoneRewardEntry"] = custom_field(
        metadata={"type": """list[DestinyMilestoneRewardEntry]"""}
    )
    reward_category_hash: int = custom_field()


@custom_define()
class DestinyMilestoneRewardEntry(BaseModel):
    """
    The character-specific data for a milestone's reward entry. See DestinyMilestoneDefinition for more information about Reward Entries.

    None
    Attributes:
        earned: If TRUE, the player has earned this reward.
        redeemed: If TRUE, the player has redeemed/picked up/obtained this reward. Feel free to alias this to "gotTheShinyBauble" in your own codebase.
        reward_entry_hash: The identifier for the reward entry in question. It is important to look up the related DestinyMilestoneRewardEntryDefinition to get the static details about the reward, which you can do by looking up the milestone's DestinyMilestoneDefinition and examining the DestinyMilestoneDefinition.rewards[rewardCategoryHash].rewardEntries[rewardEntryHash] data.
    """

    earned: bool = custom_field()
    redeemed: bool = custom_field()
    reward_entry_hash: int = custom_field()


@custom_define()
class DestinyMilestoneContent(BaseModel):
    """
    Represents localized, extended content related to Milestones. This is intentionally returned by a separate endpoint and not with Character-level Milestone data because we do not put localized data into standard Destiny responses, both for brevity of response and for caching purposes. If you really need this data, hit the Milestone Content endpoint.

    None
    Attributes:
        about: The "About this Milestone" text from the Firehose.
        item_categories: If DPS has defined items related to this Milestone, they can categorize those items in the Firehose. That data will then be returned as item categories here.
        status: The Current Status of the Milestone, as driven by the Firehose.
        tips: A list of tips, provided by the Firehose.
    """

    about: str = custom_field()
    item_categories: list["DestinyMilestoneContentItemCategory"] = custom_field(
        metadata={"type": """list[DestinyMilestoneContentItemCategory]"""}
    )
    status: str = custom_field()
    tips: list[str] = custom_field(metadata={"type": """list[str]"""})


@custom_define()
class DestinyMilestoneContentItemCategory(BaseModel):
    """
    Part of our dynamic, localized Milestone content is arbitrary categories of items. These are built in our content management system, and thus aren't the same as programmatically generated rewards.

    None
    Attributes:
        item_hashes: _No description given by bungie._
        title: _No description given by bungie._
    """

    item_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    title: str = custom_field()


@custom_define()
class DestinyPublicMilestone(BaseModel):
    """
    Information about milestones, presented in a character state-agnostic manner. Combine this data with DestinyMilestoneDefinition to get a full picture of the milestone, which is basically a checklist of things to do in the game. Think of this as GetPublicAdvisors 3.0, for those who used the Destiny 1 API.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activities: _No description given by bungie._
        available_quests: A milestone not need have even a single quest, but if there are active quests they will be returned here.
        end_date: If known, this is the date when the Milestone will expire/recycle/end.
        milestone_hash: The hash identifier for the milestone. Use it to look up the DestinyMilestoneDefinition for static data about the Milestone.
        order: Used for ordering milestones in a display to match how we order them in BNet. May pull from static data, or possibly in the future from dynamic information.
        start_date: If known, this is the date when the Milestone started/became active.
        vendor_hashes: Sometimes milestones - or activities active in milestones - will have relevant vendors. These are the vendors that are currently relevant. Deprecated, already, for the sake of the new "vendors" property that has more data. What was I thinking.
        vendors: This is why we can't have nice things. This is the ordered list of vendors to be shown that relate to this milestone, potentially along with other interesting data.
        manifest_milestone_hash: Manifest information for `milestone_hash`
    """

    activities: list["DestinyPublicMilestoneChallengeActivity"] = custom_field(
        metadata={"type": """list[DestinyPublicMilestoneChallengeActivity]"""}
    )
    available_quests: list["DestinyPublicMilestoneQuest"] = custom_field(
        metadata={"type": """list[DestinyPublicMilestoneQuest]"""}
    )
    end_date: datetime = custom_field()
    milestone_hash: int = custom_field()
    order: int = custom_field()
    start_date: datetime = custom_field()
    vendor_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    vendors: list["DestinyPublicMilestoneVendor"] = custom_field(
        metadata={"type": """list[DestinyPublicMilestoneVendor]"""}
    )
    manifest_milestone_hash: Optional["DestinyMilestoneDefinition"] = custom_field(default=None)


@custom_define()
class DestinyPublicMilestoneQuest(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity: A milestone need not have an active activity, but if there is one it will be returned here, along with any variant and additional information.
        challenges: For the given quest there could be 0-to-Many challenges: mini quests that you can perform in the course of doing this quest, that may grant you rewards and benefits.
        quest_item_hash: Quests are defined as Items in content. As such, this is the hash identifier of the DestinyInventoryItemDefinition that represents this quest. It will have pointers to all of the steps in the quest, and display information for the quest (title, description, icon etc) Individual steps will be referred to in the Quest item's DestinyInventoryItemDefinition.setData property, and themselves are Items with their own renderable data.
        manifest_quest_item_hash: Manifest information for `quest_item_hash`
    """

    activity: "DestinyPublicMilestoneActivity" = custom_field()
    challenges: list["DestinyPublicMilestoneChallenge"] = custom_field(
        metadata={"type": """list[DestinyPublicMilestoneChallenge]"""}
    )
    quest_item_hash: int = custom_field()
    manifest_quest_item_hash: Optional["DestinyMilestoneDefinition"] = custom_field(default=None)


@custom_define()
class DestinyPublicMilestoneActivity(BaseModel):
    """
    A milestone may have one or more conceptual Activities associated with it, and each of those conceptual activities could have a variety of variants, modes, tiers, what-have-you. Our attempts to determine what qualifies as a conceptual activity are, unfortunately, janky. So if you see missing modes or modes that don't seem appropriate to you, let us know and I'll buy you a beer if we ever meet up in person.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_hash: The hash identifier of the activity that's been chosen to be considered the canonical "conceptual" activity definition. This may have many variants, defined herein.
        activity_mode_hash: The hash identifier of the most specific Activity Mode under which this activity is played. This is useful for situations where the activity in question is - for instance - a PVP map, but it's not clear what mode the PVP map is being played under. If it's a playlist, this will be less specific: but hopefully useful in some way.
        activity_mode_type: The enumeration equivalent of the most specific Activity Mode under which this activity is played.
        modifier_hashes: The activity may have 0-to-many modifiers: if it does, this will contain the hashes to the DestinyActivityModifierDefinition that defines the modifier being applied.
        variants: Every relevant variation of this conceptual activity, including the conceptual activity itself, have variants defined here.
        manifest_activity_hash: Manifest information for `activity_hash`
        manifest_activity_mode_hash: Manifest information for `activity_mode_hash`
    """

    activity_hash: int = custom_field()
    activity_mode_hash: int = custom_field()
    activity_mode_type: int = custom_field()
    modifier_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    variants: list["DestinyPublicMilestoneActivityVariant"] = custom_field(
        metadata={"type": """list[DestinyPublicMilestoneActivityVariant]"""}
    )
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)
    manifest_activity_mode_hash: Optional["DestinyActivityModeDefinition"] = custom_field(default=None)


@custom_define()
class DestinyPublicMilestoneActivityVariant(BaseModel):
    """
    Represents a variant of an activity that's relevant to a milestone.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_hash: The hash identifier of this activity variant. Examine the activity's definition in the Manifest database to determine what makes it a distinct variant. Usually it will be difficulty level or whether or not it is a guided game variant of the activity, but theoretically it could be distinguished in any arbitrary way.
        activity_mode_hash: The hash identifier of the most specific Activity Mode under which this activity is played. This is useful for situations where the activity in question is - for instance - a PVP map, but it's not clear what mode the PVP map is being played under. If it's a playlist, this will be less specific: but hopefully useful in some way.
        activity_mode_type: The enumeration equivalent of the most specific Activity Mode under which this activity is played.
        manifest_activity_hash: Manifest information for `activity_hash`
        manifest_activity_mode_hash: Manifest information for `activity_mode_hash`
    """

    activity_hash: int = custom_field()
    activity_mode_hash: int = custom_field()
    activity_mode_type: int = custom_field()
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)
    manifest_activity_mode_hash: Optional["DestinyActivityModeDefinition"] = custom_field(default=None)


@custom_define()
class DestinyPublicMilestoneChallenge(BaseModel):
    """
    A Milestone can have many Challenges. Challenges are just extra Objectives that provide a fun way to mix-up play and provide extra rewards.

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_hash: IF the Objective is related to a specific Activity, this will be that activity's hash. Use it to look up the DestinyActivityDefinition for additional data to show.
        objective_hash: The objective for the Challenge, which should have human-readable data about what needs to be done to accomplish the objective. Use this hash to look up the DestinyObjectiveDefinition.
        manifest_activity_hash: Manifest information for `activity_hash`
        manifest_objective_hash: Manifest information for `objective_hash`
    """

    activity_hash: int = custom_field()
    objective_hash: int = custom_field()
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)
    manifest_objective_hash: Optional["DestinyObjectiveDefinition"] = custom_field(default=None)


@custom_define()
class DestinyPublicMilestoneChallengeActivity(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        activity_hash: _No description given by bungie._
        boolean_activity_options: The set of activity options for this activity, keyed by an identifier that's unique for this activity (not guaranteed to be unique between or across all activities, though should be unique for every *variant* of a given *conceptual* activity: for instance, the original D2 Raid has many variant DestinyActivityDefinitions. While other activities could potentially have the same option hashes, for any given D2 base Raid variant the hash will be unique). As a concrete example of this data, the hashes you get for Raids will correspond to the currently active "Challenge Mode". We have no human readable information for this data, so it's up to you if you want to associate it with such info to show it.
        challenge_objective_hashes: _No description given by bungie._
        loadout_requirement_index: If returned, this is the index into the DestinyActivityDefinition's "loadouts" property, indicating the currently active loadout requirements.
        modifier_hashes: If the activity has modifiers, this will be the list of modifiers that all variants have in common. Perform lookups against DestinyActivityModifierDefinition which defines the modifier being applied to get at the modifier data. Note that, in the DestiyActivityDefinition, you will see many more modifiers than this being referred to: those are all *possible* modifiers for the activity, not the active ones. Use only the active ones to match what's really live.
        phase_hashes: The ordered list of phases for this activity, if any. Note that we have no human readable info for phases, nor any entities to relate them to: relating these hashes to something human readable is up to you unfortunately.
        manifest_activity_hash: Manifest information for `activity_hash`
    """

    activity_hash: int = custom_field()
    boolean_activity_options: dict[int, bool] = custom_field(metadata={"type": """dict[int, bool]"""})
    challenge_objective_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    loadout_requirement_index: int = custom_field()
    modifier_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    phase_hashes: list[int] = custom_field(metadata={"type": """list[int]"""})
    manifest_activity_hash: Optional["DestinyActivityDefinition"] = custom_field(default=None)


@custom_define()
class DestinyPublicMilestoneVendor(BaseModel):
    """
    _No description given by bungie._

    Tip: Manifest Information
        This model has some attributes which can be filled with additional information found in the manifest (`manifest_...`).
        Without additional work, these attributes will be `None`, since they require additional requests and database lookups.

        To fill the manifest dependent attributes, either:

        - Run `await ThisClass.fetch_manifest_information()`, see [here](/API Reference/Models/base)
        - Set `Client.always_return_manifest_information` to `True`, see [here](/API Reference/client)

    Attributes:
        preview_item_hash: If this vendor is featuring a specific item for this event, this will be the hash identifier of that item. I'm taking bets now on how long we go before this needs to be a list or some other, more complex representation instead and I deprecate this too. I'm going to go with 5 months. Calling it now, 2017-09-14 at 9:46pm PST.
        vendor_hash: The hash identifier of the Vendor related to this Milestone. You can show useful things from this, such as thier Faction icon or whatever you might care about.
        manifest_preview_item_hash: Manifest information for `preview_item_hash`
        manifest_vendor_hash: Manifest information for `vendor_hash`
    """

    preview_item_hash: int = custom_field()
    vendor_hash: int = custom_field()
    manifest_preview_item_hash: Optional["DestinyInventoryItemDefinition"] = custom_field(default=None)
    manifest_vendor_hash: Optional["DestinyVendorDefinition"] = custom_field(default=None)
